import random
import os
import time
import keyboard
from colorama import init, Fore

init(autoreset=True)  # Инициализация colorama

class AntEaterGame:
    def __init__(self, width, height):
        self.width = width
        self.height = height
        self.anteater = {'x': width // 2, 'y': height // 2, 'symbol': '^'}
        self.ants = []
        self.anthills = []
        self.walls = []
        self.traps = []
        self.score = 0
        self.trap_timer = time.time()

        self.messages = {
            'score': f'Очки: {self.score}',
            'trap_timer': f'Таймер ловушки: {int(5 - (time.time() - self.trap_timer))} сек'
        }

    def create_anthills(self, num_anthills):
        for _ in range(num_anthills):
            x = random.randint(1, self.width - 2)
            y = random.randint(1, self.height - 2)
            self.anthills.append({'x': x, 'y': y, 'symbol': 'O'})

    def create_ants(self, num_ants):
        for _ in range(num_ants):
            anthill = random.choice(self.anthills)
            x = anthill['x'] + random.randint(-1, 1)
            y = anthill['y'] + random.randint(-1, 1)
            self.ants.append({'x': x, 'y': y, 'symbol': '*'})

    def create_outer_walls(self):
        for x in range(self.width):
            self.walls.append({'x': x, 'y': 0, 'symbol': '#'})
            self.walls.append({'x': x, 'y': self.height - 1, 'symbol': '#'})
        for y in range(self.height):
            self.walls.append({'x': 0, 'y': y, 'symbol': '#'})
            self.walls.append({'x': self.width - 1, 'y': y, 'symbol': '#'})

        for i in range(self.width // 5):
            self.walls.remove({'x': i * 5, 'y': 0, 'symbol': '#'})
            self.walls.remove({'x': i * 5, 'y': self.height - 1, 'symbol': '#'})

    def create_walls(self, num_walls):
        for _ in range(num_walls):
            x = random.randint(1, self.width - 2)
            y = random.randint(1, self.height - 2)
            self.walls.append({'x': x, 'y': y, 'symbol': '#'})

    def draw_game(self):
        output = []

        for y in range(self.height):
            row = ''
            for x in range(self.width):
                if x == 0 or x == self.width - 1:
                    row += '|'
                elif y == 0 or y == self.height - 1:
                    row += '-'
                elif x == self.anteater['x'] and y == self.anteater['y']:
                    row += Fore.RED + self.anteater['symbol']
                elif any(ant['x'] == x and ant['y'] == y for ant in self.ants):
                    row += Fore.YELLOW + self.ants[0]['symbol']
                elif any(anthill['x'] == x and anthill['y'] == y for anthill in self.anthills):
                    row += Fore.GREEN + 'O'
                elif any(wall['x'] == x and wall['y'] == y for wall in self.walls):
                    row += Fore.CYAN + '#'
                elif any(trap['x'] == x and trap['y'] == y for trap in self.traps):
                    row += Fore.MAGENTA + 'T'
                else:
                    row += ' '
            output.append(row)

        output.append(Fore.WHITE + f"Очки: {self.score}")  # Обновляем сообщение о текущем счете
        trap_timer_remaining = max(0, int(5 - (time.time() - self.trap_timer)))
        output.append(Fore.WHITE + f"Таймер ловушки: {trap_timer_remaining} сек")

        os.system('cls' if os.name == 'nt' else 'clear')  # Очистка экрана
        print('\n'.join(output))

    def move_anteater(self, direction):
        next_x, next_y = self.anteater['x'], self.anteater['y']

        if direction == 'up':
            next_y -= 1
        elif direction == 'down':
            next_y += 1
        elif direction == 'left':
            next_x -= 1
        elif direction == 'right':
            next_x += 1

        if not any(wall['x'] == next_x and wall['y'] == next_y for wall in self.walls):
            self.anteater['x'], self.anteater['y'] = next_x, next_y

    def update_ants(self):
        for ant in self.ants:
            ant['x'] += random.randint(-1, 1) // 2  # Половина шага
            ant['y'] += random.randint(-1, 1) // 2  # Половина шага

    def create_trap(self):
        self.traps.append({'x': self.anteater['x'], 'y': self.anteater['y']})
        self.trap_timer = time.time()

    def deploy_trap(self):
        trap_timer_remaining = 5 - (time.time() - self.trap_timer)
        if keyboard.is_pressed('space') and trap_timer_remaining <= 0:
            self.create_trap()

    def check_collisions(self):
        for ant in self.ants:
            if ant['x'] == self.anteater['x'] and ant['y'] == self.anteater['y']:
                self.score += 1
                self.ants.remove(ant)

        for trap in self.traps:
            for ant in self.ants:
                if ant['x'] == trap['x'] and ant['y'] == trap['y']:
                    self.score += 1
                    self.ants.remove(ant)
                    self.traps.remove(trap)

        for ant in self.ants:
            if ant['x'] == 1 or ant['x'] == self.width - 2 or ant['y'] == 1 or ant['y'] == self.height - 2:
                self.ants.remove(ant)

    def run(self):
        print("Добро пожаловать в игру 'Ловкий муравьед'!")
        print("Цель игры:")
        print("   - Поедать муравьев, которые появляются из муравейников.")
        print("   - Избегать столкновения с муравьями, стенами и границами поля.")
        print("   - Размещать ловушки для захвата муравьев.")
        print()
        print("Управление:")
        print("   - Используйте клавиши стрелок для перемещения муравьеда.")
        print("   - Нажмите Пробел, чтобы разместить ловушку (раз в 5 секунд).")
        print()
        print("Информация:")
        print("   - Муравьи хаотично перемещаются по полю.")
        print("   - Если муравей достигает границы поля, он исчезает.")
        print("   - Если муравей соприкасается с муравьедом, он считается пойманным, и вы получаете 1 очко.")
        print("   - Игра завершается, когда все муравьи съедены или упущены.")
        print()
        input("Нажмите Enter, чтобы начать игру...")

        self.create_outer_walls()
        self.create_anthills(4)
        self.create_ants(10)
        self.create_walls(5)

        while True:
            self.draw_game()

            if keyboard.is_pressed('q'):
                break
            elif keyboard.is_pressed('up'):
                self.move_anteater('up')
            elif keyboard.is_pressed('down'):
                self.move_anteater('down')
            elif keyboard.is_pressed('left'):
                self.move_anteater('left')
            elif keyboard.is_pressed('right'):
                self.move_anteater('right')

            self.update_ants()
            self.deploy_trap()
            self.check_collisions()

            if not self.ants:
                break

            time.sleep(0.1)

        if self.score == 0:
            input(Fore.WHITE + "Игра окончена! Вы проиграли. Нажмите Enter для выхода.")
        else:
            input(Fore.WHITE + f"Игра окончена! Победа! Ваши очки: {self.score}. Нажмите Enter для выхода.")

if __name__ == "__main__":
    game = AntEaterGame(width=30, height=20)
    game.run()
